#!/usr/bin/perl -w
#
# Here is the piecemeal code we wrote that will parse some fits files, and
# not others.  Just deposit it in the same directory as your fits files
# and 'chmod u+x fitswf.pl' (or whatever name you give it) and then
# try running it with 'fitswf.pl -l filename.fits'  If that doesn't work,
# try using -b instead.  If that doesn't work, who knows?
#
# Rollin
#
#---------------------------------------------------------------------------
#
# Usage: fitswf.pl [-l|-b] filename.fits
#
# If -l option doesn't work, try the -b option.  If that doesn't work, then
# you will have to try something else.
#
# Rollin C. Thomas       - Darrin Casebeer          - 6 Jan 2000
# thomas@mail.nhn.ou.edu - casebeer@mail.nhn.ou.edu
#
use strict;
my ($bo,$fn)=argsetup(@ARGV);exit unless defined($fn);
my ($waxisr,$faxisr)=fits_dataread($bo,$fn);
for(my $i=0;$i<scalar(@$waxisr);$i++) { print "$$waxisr[$i] $$faxisr[$i]\n"; }

sub fits_dataread
{
 # Opens the file, reads the header and then uses CRVAL1 and CDELT1 to read
 # the wavelength table, then return it as arrayrefs.  Make sure you have
 # the correct byte order.
 # Usage:   fits_dataread("-b",$fn) for big-endian.
 #          fits_dataread("-l",$fn) for little-endian.
 # Returns: \@waxis,\@faxis (wavelength axis, flux axis).
 my ($bo,$fn)=@_;my %parm=();my $eps=1.0e-50;
 my @waxis=();my @faxis=();
 open(FH,"$fn") || die "No such file.\n";
 binmode(FH);
 for(my $i=0;$i<216;$i++)
 {
  read(FH,my $buff,80);
# print "$buff\n";
  if($buff=~/^CRVAL1|^CDELT1/)
  {
   $buff=~s/ .//g;
   my @line=split(/=/,$buff);
   $parm{$line[0]}=$line[1];
  }
 }
 my $i=0;
 if($bo eq "-l")
 {
# while(read(FH,my $buff,8))
  while(read(FH,my $buff,4))
  {
   my $wave=$i*$parm{"CDELT1"}+$parm{"CRVAL1"};
   my $temp=unpack("f",$buff);
   print "$wave $temp\n";
   if($temp>$eps) {push(@waxis,$wave);
   push(@faxis,$temp);}
   $i++;
  }
 } else {
# while(read(FH,my $buff,8))
  while(read(FH,my $buff,4))
  {
   my $wave=$i*$parm{"CDELT1"}+$parm{"CRVAL1"};
   my $temp=unpack("f",pack("LL",reverse(unpack("N2",$buff))));
   if($temp>$eps) {push(@waxis,$wave);push(@faxis,$temp);}
#  print "$wave $temp\n";
   $i++;
  }
 }
 close(FH);
 return (\@waxis,\@faxis);
}

sub byteorder
{
 # Determines the host's byte order and prints it to stdout.
 use Config;
 use Sys::Hostname;
 print hostname()." uses ";
 if($Config{byteorder}=~/4321/) { print "big-endian.\n";    }
 else                           { print "little-endian.\n"; }
}

sub argsetup
{
 # Make things nice and modular for the benefit of our friends.
 my @ret=();
 if(scalar(@_)==0) { byteorder(); return; }
 if(scalar(@_)==1) { $ret[0]="-l";  $ret[1]=$_[0]; }
 if(scalar(@_)==2) { $ret[0]=$_[0]; $ret[1]=$_[1]; }
 if($ret[0] ne "-l" && $ret[0] ne "-b"){print "No such argument.\n";return;}
 return @ret;
}

