!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! Copyright D.J. Jeffery, 2006jan01.
!
!  specdif.f is for using the DIFF1 test on two sets of SN spectra.
!  The sets can be defined by the using in a namelist or one or both
!  can be the SUSPEND set of SN spectra.
!
!  This is the version 2.0 of specdif.  Version 1.0 is now specdif1.f.
!
!   References:
!
!     \bibitem[Jeffery et al.(2007)]{jeffery2007b} Jeffery, D. J., Ketchum, W.,
!          Branch, D., Baron, E., Elmhamdi, A., \&~Danziger, I. J.,
!          2007, ApJS, in press, astro-ph/0607084
!          % <a href="http://arxiv.org/abs/astro-ph/0607084">
!          % Goodness-of-Fit Tests DIFF1 and DIFF2 for Locally-Normalized 
!          % Supernova Spectra</a>.
!          % Local normalization and DIFF1/2.
!
!     \bibitem[Metcalf et al.(2004)]{metcalf} Metcalf, M., Reid, J., 
!          \&~Cohen, M. 2004,
!          Fortran 95/2003 Explained
!          (Oxford:  Oxford University Press) (MRC)
!          % Pretty good reference book.
!
!     \bibitem[Press et al.(1992)]{press1992} Press, W. H., Teukolsky, S. A.,
!          Vetterling, W. T., \& Flannery, B. P. 1992,
!            Numerical Recipes in Fortran
!            (Cambridge:  Cambridge University Press)
!            % Bill should have given me a complimentary copy when
!            % I co-taught with him in 1993!
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
      module specdif_mod
      use numerical2_mod
      implicit none
!
      integer, parameter :: ntype=7      ! For now just 6 types Ia, II, IIb,
!                                        !   Ib, Ic, and Ic_hyp.
!                                        !   0 is the all-types category.
!                                        !   One could get more finicky and have
!                                        !   subtypes II-P, II-L, IIn, and Ibc,
!                                        !   but I don't think those
!                                        !   distinctions
!                                        !   will be useful for some time.
!
      character :: atype(0:ntype)*6=(/'      ',                          &
!
     &                                'Ia    ',                          &
     &                                'II    ',                          &
     &                                'IIb   ',                          &
!
     &                                'Ib    ',                          &
     &                                'Ibc   ',                          &
     &                                'Ic    ',                          &
!
     &                                'Ic_hyp'/)
!
      integer :: itype_specify(0:ntype)                                  &
     &                          =(/1,  1,1,1,  1,1,1, 1/) ! Default DIFF1 test
!                                                         !   will include all
!                                                         !   supernova types. 
!
      real (kind=nprecision), allocatable :: yy(:)
!
      end module specdif_mod
    
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!-----------------------------------------------------------------------
!
      program specdif
      use numerical2_mod
      use physical2_mod
      use specnamelist_mod
      use specdif_mod
      implicit none
!
      integer, parameter :: nfile=500   ! Enough for the foreseeable future. 
      integer, parameter :: ndiffdis=10
!
      character :: afile2*180='tmpdif'   ! Default name for the output file.
      character :: atmp*180,atmp1*180,atmp2*180
      real (kind=nprecision) :: diff(nfile,nfile)=0.d0
      real (kind=nprecision) :: diff_weight(nfile,nfile)=0.d0
      real (kind=nprecision) :: diff1
      real (kind=nprecision) :: diff1_invalid1=1.d3
      real (kind=nprecision) :: diff1_invalid2=1.d4
      real (kind=nprecision) :: diff1st(2)
      real (kind=nprecision) :: diff2nd(2)
      real (kind=nprecision) :: diffdis_incr=0.1d0
      real (kind=nprecision) :: diffmax
      real (kind=nprecision) :: diffmin
      real (kind=nprecision) :: epoch1(nfile)=0.d0
      real (kind=nprecision) :: epoch2(nfile)=0.d0 
      real (kind=nprecision) ::                                             &
     & epochlim(2)=(/-1.d+30,1.d+30/)  ! These are effectively infinite limits.
      integer :: i,j,k,l,m,n
      integer :: ia,ib
      integer :: i1a,i1b
      integer :: i2a,i2b
      integer :: icompare_mode=4       ! 4 is default.
      integer :: idirection
      integer :: idiffnum 
      integer :: idiffdis(ndiffdis)
      integer :: iend
      integer :: ipower_dif=1          ! 1 is default and standard value. 
!                                      ! 2 which makes DIFF1 more like a 
!                                      !   chi-square is a plausible 
!                                      !   alternative.
!                                      ! ipower_dif is an obsolete parameter
!                                      !   for the time being.
      integer :: isnfile1=0
      integer :: isnfile1_type(nfile)=0   ! 0 is type unspecified. 
      integer :: isnfile2=0
      integer :: isnfile2_type(nfile)=0   ! 0 is type unspecified.
      integer :: ispecified
      integer :: iorder(nfile)
      integer :: iout1(nfile)=0
      integer :: istart
      integer :: isynthetic=0
      integer :: itmp
      integer :: itmp1
      integer :: itype
      integer :: itype_count(0:ntype)=0
      integer :: itype_count_sp(0:ntype)=0
      integer :: jorder 
      character :: record*180
      real (kind=nprecision) :: smallest 
      character :: snfile1(nfile)*180
      character :: snfile2(nfile)*180
      character :: snlist*180='list.dat'  ! This is my default name for this
!                                         !   file created by ls -lR > list.dat.
      character :: snpath*180='.'   ! This is my default path for
!                                   !   SUSPEND directory tree. 
      character :: specdif_par*180='specdif.par' ! This is my default name 
!                                                  ! the parameter file. 
      real (kind=nprecision) :: tmp,tmp1,tmp2
      real (kind=nprecision) :: weight
      real (kind=nprecision) :: weight_sum(2)
      real (kind=nprecision) :: weight_sumsq(2)
      real (kind=nprecision) :: weight_limit=.2d0  ! Seems OK.
      real (kind=nprecision) :: wlam1(2,nfile)=0.d0
      real (kind=nprecision) :: wlam2(2,nfile)=0.d0
      real (kind=nprecision) :: x 
      real (kind=nprecision), allocatable :: yy1(:),yy2(:)
!
!-----------------------------------------------------------------------
!
      namelist/specdif_namelist/                                         &
     & afile2,                                                            &
     & epochlim,                                                          &
     & icompare_mode,                                                     &
     & ipower_dif,                                                        &
     & isnfile1,snfile1,epoch1,wlam1,isnfile1_type,                       &
     & isnfile2,snfile2,epoch2,wlam2,isnfile2_type,                       &
     & isynthetic,                                                        &
     & itype_specify,                                                     &
     & snlist,                                                            & 
     & snpath,                                                            &
     & specdif_par
!
      namelist/specdifb/                                                  &
     & afile2,                                                            &
     & epochlim,                                                          &
     & icompare_mode,                                                     &
     & ipower_dif,                                                        &
     & isnfile1,                                                          &
     & isnfile2,                                                          &
     & isynthetic,                                                        &
     & itype_specify,                                                     &
     & snlist,                                                            &
     & snpath,                                                            &
     & specdif_par
!
!-----------------------------------------------------------------------
      write(*,*)
      write(*,*) 'In specdif, before doing anything.'
!      stop
!-----------------------------------------------------------------------
!
      iprecision=precision(1.d0)   ! iprecision is in module numerical2_mod.
      smallest=10.d0**(-iprecision)
!
!-----------------------------------------------------------------------
!
      itmp1=0 
  110 continue
      open(unit=1,file=specdif_par,status='old',action='read')
        call readcopy(1,0,0,'&specdif_namelist',itmp)
        backspace 1
        read(1,specdif_namelist)
        if(trim(specdif_par) .ne. 'specdif.par'  .and.                   &
     &     len_trim(specdif_par) .ne. 0           .and.                  &
     &     itmp1 .eq. 0                                  ) then
           close(unit=1)
           itmp1=1
           goto 110
        end if
        open(unit=10,file=afile2,status='unknown',action='write')
        write(*,specdifb)
        write(10,specdifb)
        if(icompare_mode .eq. 1) then
            write(10,*)
            write(10,'(a)') (trim(snfile1(i)),i=1,isnfile1)
            write(10,*)
            write(10,'(a)') (trim(snfile2(i)),i=1,isnfile2)
          else if(icompare_mode .eq. 2) then
            write(10,'(a)') (trim(snfile1(i)),i=1,isnfile1)
          else if(icompare_mode .eq. 3) then
            write(10,'(a)') (trim(snfile2(i)),i=1,isnfile2)
         end if
      close(unit=1)
!
      if(mod(icompare_mode,2) .eq. 0) then
!         --------------------------------------------------------------
          write(*,*)
          write(*,*) 'Before reading the dif file names from snlist.'
!         --------------------------------------------------------------
!
          open(unit=2,file=trim(snpath)//snlist,                          &
     &         status='old',action='read')
            i=0
            do410 : do 
              read(2,'(a)',end=120) record
!              write(*,'(a)') trim(record)
              if(index(record,'/dif:') .eq. 0) cycle do410
!              write(*,'(a)') trim(record)
!
              call id_type(trim(record),itype,ispecified)
              if(ispecified .eq. 0) cycle do410 ! The supernova doesn't
!                                               !   belong to the 
!                                               !   specified types.
!
              if(itype .ne. 0) itype_count(0)=itype_count(0)+1
              itype_count(itype)=itype_count(itype)+1
!
              atmp=trim(snpath)//record(2:index(record,'/dif:')+3)//      &
     &             '/'
!              write(*,*) 
!              write(*,*) trim(atmp)
!
!  Now we read in all the spectra for the supernova.
!
              read(2,*,end=120) record
              do420 : do
                read(2,'(a)',end=120) record
!                write(*,*) trim(record)
                if(record(1:1) .eq. ' ') cycle do410
                if(index(record,'dif') .eq. 0) cycle do420
                if(isynthetic .eq. 0  .and.                              &
     &              (index(record,'syn') .ne. 0  .or.                    &
     &               index(record,'pho') .ne. 0      )) cycle do420 
                i=i+1
!
                if(itype .ne. 0) itype_count_sp(0)=itype_count_sp(0)+1
                itype_count_sp(itype)=itype_count_sp(itype)+1
!
                j=index(record,'sn',.true.)   ! Start of file name.
                k=index(record,'f',.true.)    ! End of file name.
                snfile2(i)=trim(atmp)//trim(record(j:k))
                isnfile2_type(i)=itype
!                write(*,'(2i5,2x,a)') i,itype,trim(snfile2(i))
              end do do420
            end do do410
  120       continue
            isnfile2=i 
          close (unit=2)
!
          write(10,*)
          write(10,'(a)') 'Count of supernova types and type spectra.'
          write(10,*)
          write(10,"('Type',6x,3x,'No. of SNe',4x,'No. of Spectra')")
          write(10,*)
!
          do422 : do i=0,ntype
            if(i .eq. 0) then
                atmp='All types'
              else
                atmp=atype(i)
            end if
            write(10,"(a10,' & ',i10,' & ',i15,' \\')")                      &
     &       atmp,itype_count(i),itype_count_sp(i)
          end do do422 
!
      end if
!
      if(icompare_mode .ge. 3) then
          isnfile1=isnfile2
          snfile1(:)=snfile2(:)
          isnfile1_type(:)=isnfile2_type(:)
      end if
!
!-----------------------------------------------------------------------
      write(*,*)
      write(*,*) 'Before computing all the DIFF1 values.'
!-----------------------------------------------------------------------
!
      idiffnum=0
      diff1st=0.d0
      diff2nd=0.d0
      diffmin=1.d6
      diffmax=0.d0
      idiffdis(:)=0.d0
      weight_sum(:)=0.d0
      weight_sumsq(:)=0.d0
!
      do430 :  do i=1,isnfile1
        call specfile_read(snfile1(i),isnfile1_type(i),                     &
     &                     wlam1(1,i),wlam1(2,i),epochlim,                  &
     &                     i1a,i1b,epoch1(i))
        write(*,'(3i5,2a)') i,isnfile1_type(i),i1a,                         &
     &            ' After reading ',trim(snfile1(i))
        if(i1a .lt. 0) then
           iout1(i)=1
           cycle do430
        end if
        if(.not.allocated(yy1)) allocate(yy1(i1a:i1b))
        yy1(i1a:i1b)=yy(i1a:i1b)
!
        do440 : do j=1,isnfile2 
!
          if(icompare_mode .ge. 3) then
!
!            -----------------------------------------------------------
!            Note we don't double count for the statistics
!            and we don't count the self DIFF1 test either.
!            -----------------------------------------------------------
!
             if(j .le. i) then
                 epoch2(j)=epoch1(j) 
                 if(j .lt. i) then
                     diff(j,i)=diff(i,j)
                     diff_weight(j,i)=diff_weight(i,j)
                   else 
                     diff(j,i)=0.d0
                     diff_weight(j,i)=0.d0
                 end if
                 if(iout1(j) .eq. 1) then
                     diff(j,i)=diff1_invalid2
                     diff_weight(j,i)=0.d0
                 end if
                 cycle do440
             end if
          end if
!
          call specfile_read(snfile2(j),isnfile2_type(j),                  &
     &                       wlam2(1,j),wlam2(2,j),epochlim,               &
     &                       i2a,i2b,epoch2(j))
!          write(*,*)
!          write(*,*) 'j=',j,' After reading ',trim(snfile2(j))
          if(i2a .lt. 0) then
             diff(j,i)=diff1_invalid2  ! This means out of the allowed 
!                                      !   phase limits. 
             diff_weight(j,i)=0.d0
             cycle do440
          end if
          if(.not.allocated(yy2)) allocate(yy2(i2a:i2b))
          yy2(i2a:i2b)=yy(i2a:i2b)
!
!-----------------------------------------------------------------------
          write(*,*)
          write(*,*) 'Before the DIFF1 test.'
          write(*,*) 'The 1st and 2nd spectra are:.'
          write(*,'(i10,5x,a,f10.1)') i,trim(snfile1(i)),epoch1(i)
          write(*,'(i10,5x,a,f10.1)') j,trim(snfile2(j)),epoch2(j)
!-----------------------------------------------------------------------
!
          ia=max(i1a,i2a)
          ib=min(i1b,i2b)
          weight=exp(real(ib-ia,nprecision)*xx_del)-1.d0
          diff_weight(j,i)=weight
          if(weight .lt. weight_limit) then
              diff(j,i)=diff1_invalid1
              if(allocated(yy2)) deallocate(yy2)
              cycle do440
          end if
!
          diff1=0.d0
          tmp=0.d0
          do450 : do k=ia,ib
!            diff1=diff1+                                                  &
!     &       ( abs(yy1(k)-yy2(k))                                         & 
!     &        /max(abs(yy1(k)-1.d0),                                      &
!     &                abs(yy2(k)-1.d0),smallest)                          &
!     &        )**ipower_dif
             diff1=diff1+abs(yy1(k)-yy2(k))
             tmp=tmp+abs(yy1(k)-1.d0)+abs(yy2(k)-1.d0)
          end do do450
!
!          diff1=diff1/real(ib-ia+1,nprecision)
          diff1=diff1/(tmp)
          diff(j,i)=diff1
!
          if(allocated(yy2)) deallocate(yy2)
!
!         --------------------------------------------------------------
          write(*,'(15x,a,f20.10)') 'DIFF1=',diff(j,i)
          write(*,'(15x,a,f20.10)') 'DIFF1 weight=',diff_weight(j,i)
!         --------------------------------------------------------------
!
!          if(diff1 .le. 2.d0*ipower_dif) then
          if(diff1 .le. 1.000001d0) then
              idiffnum=idiffnum+1
              diffmin=min(diffmin,diff1)
              diffmax=max(diffmax,diff1)
              k=max(  ceiling(diff1/diffdis_incr) ,  1)
!              write(*,*) diff1,diff1/diffdis_incr,m
              idiffdis(k)=idiffdis(k)+1
              do452 : do m=1,2
                if(m .eq. 1) then
                    tmp=1.d0
                  else
                    tmp=weight
                end if
                weight_sum(m)=weight_sum(m)+tmp
                weight_sumsq(m)=weight_sumsq(m)+tmp**2
                diff1st(m)=diff1st(m)+diff1*tmp
                diff2nd(m)=diff2nd(m)+diff1**2*tmp ! See Bev-73.
              end do do452
            else if(diff1 .lt. diff1_invalid1-100.d0) then
              write(*,*) i,j
              write(*,*) trim(snfile1(i))
              write(*,*) trim(snfile2(j))
              write(*,*) diff1,diff1/diffdis_incr
              write(*,*) 'This should not happen.'
              stop 'program specdif'
          end if
        end do do440
        if(allocated(yy1)) deallocate(yy1)
      end do do430
!
      if(allocated(yy)) deallocate(yy)
!
!----------------------------------------------------------------------
          write(*,*)
          write(*,*) 'Before writing the diagnostics.'
!----------------------------------------------------------------------
!
      if(idiffnum .ge. 2) then
          if(icompare_mode .le. 2) then
              itmp=isnfile1+isnfile2
            else
              itmp=isnfile2
          end if
          write(10,*)
          write(10,*) 'Diagnostics:'
          write(10,*) 'Total spectra used',itmp 
          write(10,*) 'DIFF1 evaluations count ',idiffnum
          write(10,*) 'DIFF1 minimum ',diffmin
          write(10,*) 'DIFF1 maximum ',diffmax
          do454 : do i=1,2
            if(i .eq. 1) then
                write(10,*) 'Without weighting.'
              else
                write(10,*) 'With weighting.'
            end if
            write(*,*) i,' case for weight ',weight_sum(i)
            tmp1=diff1st(i)/weight_sum(i)
            write(10,'(5x,a,f11.7)') 'DIFF1 mean ',tmp1
!           -----------------------------------------------------------
!           See Bev-73 and Bev-19 for the standard deviation formula. 
!           -----------------------------------------------------------
            tmp2=      sqrt(diff2nd(i)/weight_sum(i)-tmp1**2)                 &
     &                 *sqrt(  real(idiffnum,nprecision)                      &
     &                        /(real(idiffnum,nprecision) - 1.d0)  )
            write(10,'(5x,a,f11.7)') 'DIFF1 standard error ',                 &
     &       tmp2*(sqrt(weight_sumsq(i))/weight_sum(i))  ! Bev-73, my notes.
            write(10,'(5x,a,f11.7)') 'DIFF1 standard deviation ',tmp2
          end do do454
!         -------------------------------------------------------------
          write(*,*) idiffnum,tmp1,tmp2,diffmin,diffmax
!         -------------------------------------------------------------
          write(10,*)
          write(10,*) 'DIFF1 frequency distribution'
          write(10,*)
          write(10,"(' Incr',5x,' Zone',10x,'Count',7x,'Fraction')")
          do456 : do i=1,ndiffdis
            write(10,'(i5,5x,f5.1,i15,f15.10)')                              &
     &      i,diffdis_incr*real(i,nprecision),                               &
     &      idiffdis(i),                                                     &
     &      real(idiffdis(i),nprecision)                                     &
     &       /real(idiffnum,nprecision)
          end do do456
      end if
!
!----------------------------------------------------------------------
      write(*,*)
      write(*,*) 'Before printing out the results for spectra.'
!----------------------------------------------------------------------
!
      if(icompare_mode .lt. 4) then
          istart=1
          iend=isnfile1
          idirection=1 
        else
!----------------------------------------------------------------------
!         Only in the case of everything against everything
!         do we want the reverse order.   This is just so that
!         the most recent supernovae appear at the top of the
!         output file. 
!----------------------------------------------------------------------
          istart=isnfile1
          iend=1
          idirection=-1 
      end if
!
!      write(*,*) 'diff(1,1)'
!      write(*,*) diff(1,1)
!
      write(10,*)
      write(10,*)
      write(10,'(2a)') 'Spectrum by spectrum DIFF1',                          &
     &                 ' and DIFF1 weight values.'
      write(10,*)
!
      do460 : do i=istart,iend,idirection
!        write(*,*) i,diff(i,i)
        if(iout1(i) .eq. 1) cycle do460
        call sortheap2(isnfile2,diff(:,i),iorder)
        write(10,*)
        atmp1=snfile1(i)(1:index(snfile1(i),'/',.true.)-1)
        atmp2=snfile1(i)(index(snfile1(i),'/',.true.)+1:)
        write(10,'(a)') '1st spectrum path and file: '//trim(atmp1)           &
     &                   //' and '//trim(atmp2) 
        write(10,*)
        write(10,"('Order',                                                   &
     &                     1x,'Type',2x,                                      &
     &                     1x,'1st spectrum file',13x,                        &
     &                     1x,'Type',2x,                                      &
     &                     1x,'2nd spectrum file',13x,                        &
     &                         1x,'1st phase',                                &
     &                         1x,'2nd phase',                                &
     &                         1x,'Rel phase',                                &
     &                         15x,'DIFF1',
     &                          8x,'DIFF1 weight')")
        do470 : do j=1,isnfile2
          jorder=iorder(j)
          write(*,*) i,j,jorder,diff(jorder,i)
          if(diff(jorder,i) .gt. diff1_invalid2-100.d0) exit do470
          atmp=snfile1(i)
          atmp1=snfile1(i)(index(atmp,'/',.true.)+1:)
          atmp=snfile2(jorder)
          atmp2=snfile2(jorder)(index(atmp,'/',.true.)+1:)
          write(10,'(i5,2(1x,a6,1x,a30),3f10.1,2f20.7)')                      &
     &    j,                                                                  &
     &    atype(isnfile1_type(i)),                                            &
     &    atmp1,                                                              &
     &    atype(isnfile2_type(jorder)),                                       &
     &    atmp2,                                                              & 
     &    epoch1(i),                                                          & 
     &    epoch2(jorder),                                                     & 
     &    epoch2(jorder)-epoch1(i),                                           & 
     &    diff(jorder,i),                                                     &
     &    diff_weight(jorder,i)              
        end do do470
      end do do460
!
      close(unit=10)
!
      end program specdif
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
      function fweight(x)
      use numerical2_mod
      implicit none
      real (kind=nprecision) :: fweight
      real (kind=nprecision) :: x
      real (kind=nprecision) :: x_fid=2.d0
!
      fweight=(1.d0-exp(-x_fid))/(1.d0-exp(-x))
!
      end function fweight
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
      subroutine specfile_read(snfile,itype,                              &
     &                         wlama,wlamb,epochlim,                      &
     &                         ia,ib,epochi)
      use numerical2_mod
      use physical2_mod
      use specnamelist_mod
      use specdif_mod
      implicit none
!
      real (kind=nprecision), intent(inout) :: epochi
      real (kind=nprecision), intent(in) :: epochlim(2)
      integer :: i,j,k,l,m,n
      integer :: i0
      integer, intent(out) :: ia
      integer, intent(out) :: ib
      integer :: idata
      integer :: ispecified
      integer, intent(inout) :: itype
      character :: record*180 
      character, intent(in) :: snfile*(*) 
      real (kind=nprecision),intent(inout) :: wlama 
      real (kind=nprecision),intent(inout) :: wlamb
      real (kind=nprecision) :: x 
      real (kind=nprecision) :: y 
!
      open(unit=1,file=snfile,status='old',action='read')
      call readcopy(1,0,0,'&specnamelist',idata)
      backspace 1
      read(1,'(a)') record
      if(index(record,'&specnamelist') .ne. 0) then
!
!          write(*,*) 'In this case, the file has namelist '//             &
!     &               'specnamelist parameters.'
!
          backspace 1
          call specnamelist_default2
          read(1,specnamelist)
          call id_type(trim(aname),itype,ispecified)
          write(*,*) 'trim(aname),itype,ispecified'
          write(*,*) trim(aname),itype,ispecified
          if(iformat .gt. 0) then
!             ! Giving the right redshift to the good limits I hope.
              wlama=wlammin3/(1.d0+v_helio*1.d+5/clight)
              wlamb=wlammax3/(1.d0+v_helio*1.d+5/clight)
            else
              wlama=wlammin3
              wlamb=wlammax3
          end if
          epochi=epoch
      end if
!
      if((itype_specify(itype) .eq. 0  .and.                             &
     &    itype_specify(0)     .eq. 0        )  .or.                     &
     &  epochi .lt. epochlim(1)                 .or.                     &
     &   epochi .gt. epochlim(2)                     ) then
          ia=-1        ! I'm using ia for different purpose than
!                      !   than in the other case, but who cares.
          goto 200     ! Eddie agrees:  forward goto's are cool.
      end if 
!
! The following two command lines are needed for when there is a header,
! but it has no namelist specnamelist.   It's more efficient to have
! them after we've checked for omission.
!
      rewind 1
      call readcopy(1,0,0,' ',idata)
!
!
! The following if-block is executed if there are not
!   prescribed wavelength limits in namelists specdif
!   or  specnamelist (which is in the file header).
!   In this case, the whole spectrum is used for DIFF1 test.
!   This is not recommended for either observed or synthetic
!   spectra, but is fine for testing or laziness purposes. 
!
      if(abs(wlamb) .lt. 1.d-6) then
         read(1,*) wlama,y
         do402 : do i=2,idata
           read(1,*) wlamb,y
         end do do402
         rewind 1
         call readcopy(1,0,0,' ',idata)
      end if
!
      ia=int(log(wlama)/xx_del)+1    ! Defining the good region of the spectrum.
      ib=int(log(wlamb)/xx_del)-1
      read(1,*) x,y
      x=log(x)                 ! I only use 2-column format now in dif files.
      i0=int(x/xx_del+.5d0)    ! The .5 should account for roundoff.
!                              !   We don't want to use ``ceiling'' here.
      backspace 1
      ia=max(ia,i0)
      if(allocated(yy)) deallocate(yy)  
      if(.not.allocated(yy)) allocate(yy(ia:ib))
!       write(*,*) 'i0,ia,ib'
!       write(*,*) i0,ia,ib
!        stop
      do410 : do i=i0,ib
        if(i .ge. ia) then
            read(1,*,err=110) x,yy(i)  ! Since the wavelength grid is standardized,
!                              !   I don't need to read it.
!            write(*,*) x,yy(i)
          else
            read(1,*)
         end if
      end do do410
      close(unit=1)
      goto 200            ! gotos are honored in programming history. 
!
  110 continue
      print*,'You have probably set the upper wavelength higher'
      print*,'than in the spectrum file.  This causes an attempt'
      print*,'to read past the end of file.'
      stop 'subroutine specfile-read' 
!
  200 continue
      end subroutine specfile_read
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
!
      subroutine id_type(aname,itype,ispecified)
      use numerical2_mod
      use specdif_mod 
      implicit none
!
      character, intent(in) :: aname*(*)
      character :: atmp*6
      integer :: i,j,k,l,m,n
      integer :: ka 
      integer :: kb
      integer :: kc
      integer, intent(out) :: itype
      integer, intent(out) :: ispecified
!
! Finding the supernova type from aname.
!
      ka=index(aname,'_I')
      kb=index(aname,' I')
      kc=index(aname,'I')
!      write(*,*) 'ka,kb,kc'
!      write(*,*) ka,kb,kc
      if(ka .ne. 0) then
          k=ka+1
          m=index(aname(k+1:),'/')-1+k
        else if(kb .ne. 0  .or.  kc .eq. 1) then
          if(kb .ne. 0) then
              k=kb+1
            else
              k=kc
          end if
          m=index(aname(k+1:),' ')-1+k
          if(m .lt. k) m=len_trim(aname)
      end if
!
      atmp='      '        !  I just feel better always knowing this
!                          !    starts with a blank assignment for sure
!                          !    in all fortran 95 versions.
      if(ka+kb .ne. 0  .or. kc .eq. 1) atmp=aname(k:m)
      atmp=adjustl(atmp)   !  Just to make sure atmp is left-justified
!                          !    in all fortran 95 versions.
! 
!      write(*,'(2i5,a25,5x,a6)') k,m,trim(aname),atmp
!
! Identifying the supernova type index. 
!
      itype=0    ! This is in case there is a supernova
!                !   type that doesn't correspond to any
!                !   of the types defined in this code.
!
      do410 : do i=0,ntype
        if(atmp .eq. atype(i)) then
            itype=i
            exit do410 
        end if
      end do do410
!
!      write(*,'(i5,5x,a6)') itype,atmp
!
! Finding if the type corresponds to the specification.
!
      if(itype_specify(itype) .ne. 0  .or.
     &   itype_specify(0) .ne. 0              ) then
          ispecified=1 
        else
          ispecified=0       ! The supernova doesn't belong to the
!                            !   specified types.
      end if
!
!      write(*,'(2i5,5x,a6)') ispecified,itype,atmp
!
      end subroutine id_type
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
