!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! Copyright D.J. Jeffery, 2006jan01.
!
!  specnamelist.f is a name module and default setter for the speclist.f and specdif.f programs.  
!
!   References:
!
!     \bibitem[Jeffery et al.(2007)]{jeffery2007b} Jeffery, D. J., Ketchum, W., Branch, D., Baron, E.,
!          Elmhamdi, A., \&~Danziger, I. J.,
!          2007, ApJS, in press, astro-ph/0607084
!          % <a href="http://arxiv.org/abs/astro-ph/0607084">
!          % Goodness-of-Fit Tests DIFF1 and DIFF2 for Locally-Normalized Supernova Spectra</a>.
!          % Local normalization and DIFF1/2.
!
!     \bibitem[Metcalf et al.(2004)]{metcalf} Metcalf, M., Reid, J., 
!          \&~Cohen, M. 2004,
!          Fortran 95/2003 Explained
!          (Oxford:  Oxford University Press) (MRC)
!          % Pretty good reference book.
!
!     \bibitem[Press et al.(1992)]{press1992} Press, W. H., Teukolsky, S. A.,
!          Vetterling, W. T., \& Flannery, B. P. 1992,
!            Numerical Recipes in Fortran
!            (Cambridge:  Cambridge University Press)
!            % Bill should have given me a complimentary copy when
!            % I co-taught with him in 1993!
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! See MRC-153,192 and the example /aalib/testnamelist.f.
!
      module specnamelist_mod
      use numerical2_mod
      implicit none
!
      integer, parameter :: nsmooth=10                    ! 10 smoothing zones should be far more 
!                                                         !  than I ever need.
      real (kind=nprecision), parameter :: xx_del=third*1.d-3  ! For some historical reason,  I chose
!                                                              !  to make the default logarithmic
!                                                              !  wavelength increment 1/3000 
!                                                              !  which is to first order in small
!                                                              !  relative wavelength is 1/3000.
!
      real (kind=nprecision), parameter :: xx_ref=0.d0  ! Since 1 Angstrom should be smaller than any observed wavelength,
!                                                       !   we take 1 A as the log10(lambda)=0. 
!
      character :: aname*40            ! Supernova/type/date 
      real (kind=nprecision) :: ebv    ! Foreground Galactic E(B-V). 
      real (kind=nprecision) :: epoch  ! Epoch in days relative to maximum light in some band or a maximum light
                                       !   estimate or proxy (such as date of discovery).
!
      integer :: idifmake=1 ! idifmake=1 means make a dif version of the spectrum in speclist.f.
!                           !   idifmake=0 causes no new dif file.  This best for plotting when
!                           !   one is trying to get the figure looking right and making a new dif
!                           !   file is slow.
!
      integer :: iformat=1  ! Original flux representation:   +/- 1 is f_lambda or log(f_lamba);
!                           !                                 +/- 2 is f_nu or log(f_nu).
!                           !   iformat .lt. 0, means that the spectrum has already been deredshifted by 
!                           !   v_helio.  Or so it is thought!!!? 
!     
      integer :: itick=1    ! The tick controler for non-LN SM:  1 is ordinary, 2 for fine in x, 
!                           !                                    3 for fine in y, 4 for fine in both.  
      integer :: itickln=3  ! The tick controler for LN SM:  1 is ordinary, 2 for fine in x, 
!                                                            3 for fine in y, 4 for fine in both.  
      real (kind=nprecision) :: power  ! A wavelength power multiplication to establish 
!                                      !   an artificial continuum for testing local normalization. 
!                                      !   I need it in specnamelist for creating the dif file.
      real (kind=nprecision) :: scale_flux    ! To rescale the flux if it is in stupid units. 
      integer :: ismooth                      ! The smoothing parameter.  0 is default
      real (kind=nprecision) :: smooth_x(nsmooth)      ! The relative wavelength smoothing region. 
      real (kind=nprecision) :: smooth_zone(nsmooth)   ! Wavelength upper limits on the smoothing zones. 
      real (kind=nprecision) :: t_rise=20.d0  ! Estimated rise time to maximum light in days.  Default is 20 days. 
      real (kind=nprecision) :: v_helio       ! Heliocentric velocity. 
      real (kind=nprecision) :: wavelength_corr     ! A wavelength correction for wrong wavelength scale. 
      real (kind=nprecision) :: ymin2=1.d0  ! Minimum y-parameter for plotting.
      real (kind=nprecision) :: ymax2=0.d0  ! Maximum y-parameter for plotting. If max .lt. min, there is a default.
      real (kind=nprecision) :: ymin3=1.d0  ! Minimum y-parameter for shifted plotting. 
      real (kind=nprecision) :: ymax3=0.d0  ! Maximum y-parameter for shifted plotting. 
!                                           !   If max .lt. min, there is a default. 
      real (kind=nprecision) :: wlammin1  ! Minimum wavelength of the file. 
      real (kind=nprecision) :: wlammax1  ! Maximum wavelength of the file. 
      real (kind=nprecision) :: wlammin2  ! Minimum wavelength good for plotting. 
      real (kind=nprecision) :: wlammax2  ! Maximum wavelength good for plotting. 
      real (kind=nprecision) :: wlammin3  ! Minimum wavelength good for shifting. 
      real (kind=nprecision) :: wlammax3  ! Maximum wavelength good for shifting. 
      real (kind=nprecision) :: xx_cont   ! The local normalization averaging region half-width.
!
! I like using one variable per line, but some compilers object to the
! number of continuation lines.
!
      namelist/specnamelist/
     &    aname,                                                          &  
     &    ebv,                                                            &  
     &    epoch,                                                          &  
     &    idifmake,                                                       &  
     &    iformat,                                                        &  
     &    itick,itickln,                                                  &  
     &    power,                                                          & 
     &    ismooth,smooth_x,smooth_zone,                                   &
     &    scale_flux,                                                     &  
     &    t_rise,                                                         &  
     &    v_helio,                                                        &  
     &    wavelength_corr,                                                &  
     &    ymin2,ymax2,                                                    &  
     &    ymin3,ymax3,                                                    &  
     &    wlammin1,wlammax1,                                              &  
     &    wlammin2,wlammax2,                                              &  
     &    wlammin3,wlammax3,                                              &  
     &    xx_cont                                                         &  
!
      end module specnamelist_mod
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! This subprogram restores the default namelist values.
! One should call it before reading a specnamelist.
!
      subroutine specnamelist_default
      use numerical2_mod
      use specnamelist_mod
      implicit none
!
      idifmake=1    ! idifmake=1 means make a dif file in specnamelist.  idifmake=0 means don't.
      iformat=1                  ! Original flux representation:   +/- 1 is f_lambda or log(f_lamba);
!                                !                                 +/- 2 is f_nu or log(f_nu).
!                                ! iformat .lt. 0, means that the spectrum has already been deredshifted by
!                                ! v_helio.  Or so it is though!!!?
      ismooth=1    ! A default smoothing for all spectra and dif files.  Some spectra may take
!                  !  a different amount:  more or less.
      itick=1      ! The tick controler for non-LN SM:  1 is ordinary, 2 for fine in x, 
!                  !                                    3 for fine in y, 4 for fine in both.  
      itickln=3    ! The tick controler for LN SM:  1 is ordinary, 2 for fine in x, 
!                  !                                3 for fine in y, 4 for fine in both.  
      power=1.d0   ! A wavelength power multiplication to establish an artificial continuum 
!                  !   for testing local normalization.  
!                  !   1.d0 defaults to no multiplication
      smooth_x=.5d0/300.d0   ! This uses a box of (1000/(3.d+5))*c=1000 km/s around the wavelength which is 
!                            !   a reasonable smoothing region for supernovae. 
      smooth_zone=1.d10      ! This is infinity which is useful for one smoothing zone.
      scale_flux=1.d0 ! To rescale the flux if it is in stupid units.
      t_rise=20.d0    ! Estimated rise time to maximum light in days.  Default is 20 days.
      v_helio=0.d0    ! Heliocentric velocity default=0.d0.
      wavelength_corr=0.d0   ! Wavelength scale correction default=0.d0.
      ymin2=1.d0      ! Minimum y-parameter for plotting default to default y-region..
      ymax2=0.d0      ! Maximum y-parameter for plotting. 
      ymin3=1.d0      ! Minimum y-parameter for shifted plotting default to default y-region.
      ymax3=0.d0      ! Maximum y-parameter for shifted plotting.
      wlammin1=0.d0   ! This will probably give rise to a nice arithmetic indefinite.
      wlammax1=0.d0   ! This will probably give rise to a nice arithmetic indefinite. 
      wlammin2=0.d0   ! This will probably give rise to a nice arithmetic indefinite. 
      wlammax2=0.d0   ! This will probably give rise to a nice arithmetic indefinite. 
      wlammin3=0.d0   ! This will probably give rise to a nice arithmetic indefinite.  
      wlammax3=0.d0   ! This will probably give rise to a nice arithmetic indefinite. 
      xx_cont=0.15d0  ! The local normalization averaging region half-width.
!                     !   This default seems good for normal supernovae and hypernovae.
!                     !   Old microdensitor transmission tracings seem to weird and need a smaller value I think.
!                     !   Note that this is the half-relative smoothing length used in speclist.f.  There are no
!                     !   logarithmic conversions as for the gridding and shifting.  Thank god.
!
      end subroutine specnamelist_default
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! This subprogram sets default namelist values for specdif2.f in the
! DIFF1 package.
!
      subroutine specnamelist_default2
      use numerical2_mod
      use specnamelist_mod
      implicit none
!
      aname=' '    !  Default is no supernova name or type.
      epoch=0.d0   !  Default phase is set to 0 or maximum light.
      iformat=-1   ! -1 is do not deredshift wlammax3 and wlammin3.
                   ! +1 is for do deredshift them.
      v_helio=0.d0    ! Heliocentric velocity default=0.d0.
      wlammin3=0.d0   ! These values imply use the whole wavelength range
      wlammax3=0.d0   ! for the DIFF1 test.
!
      end subroutine specnamelist_default2
!
!23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
!
! specnamelist_format is for reformating spectrum files. 
!
      subroutine specnamelist_format(iformat,iwlam,wlam,flux,v_helio,    &
     & wavelength_corr,scale_flux,power)
      use numerical2_mod
      use physical2_mod
      implicit none
!
!
      integer :: i,j,k,l,m,n
      integer, intent(in) :: iformat
      integer, intent(in) :: iwlam 
      real (kind=nprecision), intent(in) :: power 
      real (kind=nprecision), intent(in) :: scale_flux 
      real (kind=nprecision), intent(in) :: v_helio 
      real (kind=nprecision), intent(in) :: wavelength_corr 
!
! Some f95 compilers demand that iwlam precede its use
!   to set arrays.
      real (kind=nprecision), intent(inout) :: flux(iwlam) 
      real (kind=nprecision), intent(inout) :: wlam(iwlam) 
!
      if(abs(wavelength_corr) .gt. 1.d-3)                                & 
     &   wlam(:)=wlam(:)+wavelength_corr
!
!-----------------------------------------------------------------------
!      write(*,*) 'Deredshifting cosmologically correctly?? (CL-16).'
!-----------------------------------------------------------------------
!
!        write(*,*) wlam(1)
        if(iformat .gt. 0)                                               & 
     &   wlam(:)=wlam(:)/(1.d0+v_helio*1.d+5/clight)
!        write(*,*) wlam(1)
!
! Conversion from AB magnitude I hope.
        if(abs(iformat) .ge. 3)                                          &  
     &   flux(:)=10.d0**(-.4d0*flux(:))
!
! Conversion from f_nu to f_lambda
        if(mod(abs(iformat),2) .eq. 0)                                   &  
     &   flux(:)=flux(:)*(clight/wlam(:)**2)*1.d+8
!
! A scaling of the flux for plotting for whatever purpose.
        if(abs(scale_flux-1.d0) .gt. 1.d-6)                              & 
     &   flux(:)=flux(:)*scale_flux
!
! A power of wavelength multiplication for artificial continuum testing
! of local normalization. 
        if(abs(power-1.d0) .gt. 1.d-10) then
!           write(*,*) 'Before power ', flux(iwlam)
           flux(:)=flux(:)*wlam(:)**power
!           write(*,*) 'After power ', flux(iwlam)
        end if
!
      end subroutine specnamelist_format 
